//Includes
#include "MPC5746R.h"
#include "stdbool.h"

//Constants and macros
#define 	DRUN_MODE 	0x3

//Global variables
unsigned int Online_LBIST_Result;           //Stored -> On-Line LBIST Status Register
unsigned int Online_MBIST_Result_low;       //Stored -> On-Line MBIST End Flag Low Register
unsigned int Online_MBIST_Result_medium;    //Stored -> On-Line MBIST End Flag Medium Register
int BistCheckFlag = 0;

//Local functions
void Sys_Init(void)
{
    /* Configure RunPeripheralConfiguration registers in ME_RUN_PC0
     * Enable DRUN mode for all peripherals using ME_RUN_PC[0] */
    MC_ME->RUN_PC[MC_ME_RUN_PC0_INDEX] = MC_ME_RUN_PC_DRUN(1u);

    /* Set PLL0 as system clock
     * Enable PLL0 for DRUN mode
     * Enable PLL1 for DRUN mode
     * Enable external oscillator */
    MC_ME->DRUN_MC = MC_ME_DRUN_MC_SYSCLK(0x2)
                   | MC_ME_DRUN_MC_PLL0ON(1u)
                   | MC_ME_DRUN_MC_PLL1ON(1u)
                   | MC_ME_DRUN_MC_XOSCON(1u);

    /*****************************************************************
     * System clock dividers
     *****************************************************************/

    // Divider setup: clock will have a period ‘DIV + 1’ times

    /* Enable system clock divider /1 -> 200MHz
     * Enable divider | divide by 1
     *     0x80000000 | 0x0 */
    MC_CGM->SC_DC0 = 0x80000000 | 0x0;
//    MC_CGM->SC_DC0 = MC_CGM_SC_DC0_DE(1u)
//                   | MC_CGM_SC_DC0_DIV(0u);

    /* SXBAR clock divider = 2 -> 200MHz/2 = 100MHz
     * Enable divider | divide by 2
     *     0x80000000 | 0x10000 */
    MC_CGM->SC_DC1 = 0x80000000 | 0x10000;
//    MC_CGM->SC_DC1 = MC_CGM_SC_DC1_DE(1u)
//                   | MC_CGM_SC_DC1_DIV(1u);

    /* PBRIDGE clock divider = 4 -> 200MHz/4 = 50MHz
     * Enable divider | divide by 4
     *     0x80000000 | 0x30000 */
    MC_CGM->SC_DC2 = 0x80000000 | 0x30000;
//    MC_CGM->SC_DC2 = MC_CGM_SC_DC2_DE(1u)
//                   | MC_CGM_SC_DC2_DIV(3u);

    /**********  AUX_0 clock dividers **********/

    // Connect PLL0 to AXU_0
     MC_CGM->AC0_SC = 0x2;
//     MC_CGM->AC0_SC = MC_CGM_AC0_SC_SELCTL(0x2);

    /* PER_CLK clock divider = 3 -> 200MHz/3 = 66MHz
     * Enable divider | divide by 3
           0x80000000 | 0x20000 */
     MC_CGM->AC0_DC0 = 0x80000000 | 0x20000;
//    MC_CGM->AC0_DC0 = MC_CGM_AC0_DC0_DE(1u)
//                    | MC_CGM_AC0_DC0_DIV(2u);

    /* AC0_SC clock divider = 13 -> 200MHz/13 = 15MHz
     * Enable divider | divide by 13
     *     0x80000000 | 0xC0000 */
     MC_CGM->AC0_DC1 = 0x80000000 | 0xC0000;
//    MC_CGM->AC0_DC1 = MC_CGM_AC0_DC1_DE(1u)
//                    | MC_CGM_AC0_DC1_DIV(12u);

    /* SAR ADC clock divider = 3 -> 200MHz/3 = 66.6MHz
     * Enable divider | divide by 3
     *     0x80000000 | 0x20000 */
     MC_CGM->AC0_DC2 = 0x80000000 | 0x20000;
//    MC_CGM->AC0_DC2 = MC_CGM_AC0_DC2_DE(1u)
//                    | MC_CGM_AC0_DC2_DIV(2u);

    /* DSPI_CLK0 clock divider = 3 -> 200MHz/3 = 66.6MHz
     * Enable divider | divide by 3
     *     0x80000000 | 0x20000 */
     MC_CGM->AC0_DC3 = 0x80000000 | 0x20000;
//    MC_CGM->AC0_DC3 = MC_CGM_AC0_DC3_DE(1u)
//                    | MC_CGM_AC0_DC3_DIV(2u);

    /* DSPI_CLK1 clock divider = 3 -> 200MHz/3 = 66.6MHz
     * Enable divider | divide by 3
     *     0x80000000 | 0x20000 */
     MC_CGM->AC0_DC4 = 0x80000000 | 0x20000;
//     MC_CGM->AC0_DC4 = MC_CGM_AC0_DC4_DE(1u)
//                    | MC_CGM_AC0_DC4_DIV(2u);

    /********** AUX_1 clock dividers**********/

    /* PD LFAST PLL/BD LFAST PLL/SIPI_CLK_OUT clock divider = 8 -> 200MHz/8 = 25MHz
     * Enable divider | divide by 8
     *     0x80000000 | 0x70000 */
     MC_CGM->AC1_DC0 = 0x80000000 | 0x70000;
//    MC_CGM->AC1_DC0 = MC_CGM_AC1_DC0_DE(1u)
//                    | MC_CGM_AC1_DC0_DIV(7u);

    /********** AUX_2 clock dividers**********/

    /* SENT_CLK clock divider = 3 -> 200MHz/3 = 66.6MHz
     * Enable divider | divide by 3
     *     0x80000000 | 0x20000 */
     MC_CGM->AC2_DC0 = 0x80000000 | 0x20000;
//    MC_CGM->AC2_DC0 = MC_CGM_AC2_DC0_DE(1u)
//                    | MC_CGM_AC2_DC0_DIV(2u);

    // AUX Clock Selector 3 setup - source for PLL0 module
    // Connect (8..40MHz) XTALL to the PLL0 input 0=IRC   1=XOSC
    MC_CGM->AC3_SC = MC_CGM_AC3_SC_SELCTL(1u);

    //***AUX_4 clock dividers*****************************************//

    //AUX Clock Selector 4 setup - source for PLL1 module
    //connect PLL0 to AUX4
    MC_CGM->AC4_SC = MC_CGM_AC4_SC_SELCTL(3u);

    //***AUX_5 clock dividers*****************************************//

    MC_CGM->AC5_SC = MC_CGM_AC5_SC_SELCTL(2u);			//connect PLL0 on AUX_5

    /* eTPU clock divider = 1 -> 200MHz/1 = 200MHz
     * Enable divider | divide by 1
     *     0x80000000 | 0x00000 */
    MC_CGM->AC5_DC0 = 0x80000000 | 0x00000;
//    MC_CGM->AC5_DC0 = MC_CGM_AC5_DC0_DE(1u)
//                    | MC_CGM_AC5_DC0_DIV(0u);

    /* eMIOS clock divider = 2 -> 200MHz/2 = 100MHz
     *            Enable divider | divide by 5
     *                0x80000000 | 0x10000 */
    MC_CGM->AC5_DC1 = 0x80000000 | 0x10000;
//    MC_CGM->AC5_DC1 = MC_CGM_AC5_DC1_DE(1u)
//                    | MC_CGM_AC5_DC1_DIV(4u);

    //***AUX_6 clock dividers*****************************************//

    // CLKOUT
    //connect PPL0 to AUX_6
    MC_CGM->AC6_SC = 0x02000000;
//    MC_CGM->AC6_SC = MC_CGM_AC6_SC_SELCTL(2u);

    /* CLKOUT_0 clock divider = 20 -> 200MHz/20 = 10MHz
     * Enable divider | divide by 20
     *     0x80000000 | 0x140000 */
    MC_CGM->AC6_DC0 = 0x80000000 | 0x140000;
//    MC_CGM->AC6_DC0 = MC_CGM_AC6_DC0_DE(1u)
//                    | MC_CGM_AC6_DC0_DIV(19u);

    //***AUX_8 clock dividers*****************************************//

    /* FlexCAN clock divider = 5 -> 200MHz/5 = 40MHz
     * Enable divider | divide by 5
     *  0x80000000 | 0x40000 */
    MC_CGM->AC8_DC0 = 0x80000000 | 0x40000;
//    MC_CGM->AC8_DC0 = MC_CGM_AC8_DC0_DE(1u)
//                    | MC_CGM_AC8_DC0_DIV(4u);

    //connect PLL0 on AUX_8
    MC_CGM->AC8_SC = MC_CGM_AC8_SC_SELCTL(2u);

    //***AUX_9 clock dividers*****************************************//
    // RTI_CLK //
    // connect XTAL on AUX_9
    MC_CGM->AC9_SC = MC_CGM_AC9_SC_SELCTL(2u);

    //***AUX_10 clock dividers*****************************************//

    //connect PLL0 on AUX_10
    MC_CGM->AC10_SC = MC_CGM_AC10_SC_SELCTL(2u);

    /* TXCLK_REF_CLK clock divider = 5 -> 200MHz/4 = 50MHz
     * Enable divider | divide by 4
     *     0x80000000 | 0x30000 */
    MC_CGM->AC10_DC0 = 0x80000000 | 0x30000;
//    MC_CGM->AC10_DC0 = MC_CGM_AC10_DC0_DE(1u)
//                     | MC_CGM_AC10_DC0_DIV(3u);

    // Set PLL0 to 200MHz with 20MHz XOSC (PLL0 VCO frequency — 600 — 1250 MHz)
    //Bypass mode with PLL0 off
    PLLDIG->PLL0CR = PLLDIG_PLL0CR_CLKCFG(0u);

    /* RFDPHI1 = 10, RFDPHI = 4, PREDIV = 2, MFD = 80 (0x50h)
     * 0x50000000 |0x00040000 |0x00002000 |0x0050
     * predefined PLL0 divider register */
    PLLDIG->PLL0DV = 0x50000000 |0x00040000 |0x00002000 |0x0050;
//    PLLDIG->PLL0DV = PLLDIG_PLL0DV_RFDPHI1(10u)
//                   | PLLDIG_PLL0DV_RFDPHI(4u)
//                   | PLLDIG_PLL0DV_PREDIV(2u)
//                   | PLLDIG_PLL0DV_MFD(80u);

    // Set PLL1 to 200MHz with PLL0_PHI (20MHz) as input (PLL1 VCO frequency — 600 — 1250 MHz)
    //PLL1 on
    PLLDIG->PLL1CR = PLLDIG_PLL1CR_CLKCFG(0u);

    //predefined PLL0 divider register
    //RFDPHI = 4, MFD = 40 (0x28h)
    //0x00040000 |0x0028
    PLLDIG->PLL1DV = 0x00040000 |0x0028;
//    PLLDIG->PLL1DV = PLLDIG_PLL1DV_RFDPHI(4u)
//                   | PLLDIG_PLL1DV_MFD(40u);

    //Mode transition to apply the PLL0 setup and set Normal mode with PLL running
    //Reference manual: section 62.3.2, page 2620
    //DRUN Mode & Key: 0x30005AF0;
    MC_ME->MCTL = 0x30005AF0;
//    MC_ME->MCTL = MC_ME_MCTL_TARGET_MODE(3u)
//                | MC_ME_MCTL_KEY(0x5AF0);
    //DRUN Mode & Key: 0x3000A50F
    MC_ME->MCTL = 0x3000A50F;
//    MC_ME->MCTL = MC_ME_MCTL_TARGET_MODE(3u)
//                | MC_ME_MCTL_KEY(0xA50F);

    //ME_GS Wait for PLL stabilization
    while(MC_ME->GS && MC_ME_GS_S_PLL0(1u) != MC_ME_GS_S_PLL0(1u));
    //Waiting for end of transaction
    while(MC_ME->GS && MC_ME_GS_S_MTRANS(1u) != MC_ME_GS_S_MTRANS(1u));
    // ME_GS Check DRUN mode has successfully been entered
    while(MC_ME->GS && MC_ME_GS_S_CURRENT_MODE(DRUN_MODE) != MC_ME_GS_S_CURRENT_MODE(DRUN_MODE));
}//Sys_Init

void SIUL2_Init(void)
{
    //PJ1 as CLK_OUT (on EVB it is PJ1)
    //0x22800007
    SIUL2->MSCR[145] = 0x22800007;
}//SUIL2_Init

/*Function STCU2_Online_BIST_check -> ONLINE RESULTS CHECK
STCU2 On-Line LBIST Status Register (STCU2_LBSSW)
STCU2 On-Line LBIST End Flag Register (STCU2_LBESW)
STCU2 On-Line MBIST Status Low Register (STCU2_MBSLSW)
STCU2 On-Line MBIST Status Medium Register (STCU2_MBSMSW)
STCU2 On-Line MBIST End Flag Low Register (STCU2_MBELSW)
STCU2 On-Line MBIST End Flag Medium Register (STCU2_MBEMSW)
STCU2 On-Line LBIST MISR Read Low Register (STCU2_LB_MISRRLSWn)
STCU2 On-Line LBIST MISR Read High Register (STCU2_LB_MISRRHSWn)
*/
void STCU2_Online_BIST_check (void)
{
//Read On-Line LBIST END Flag register (LBESW)to determine if we have completed On-line LBIST
    if (STCU2->LBESW != 0){
//    if (STCU2->LBESW != STCU2_LBESW_LBESW0(0u) &&
//        STCU2->LBESW != STCU2_LBESW_LBESW1(0u) &&
//        STCU2->LBESW != STCU2_LBESW_LBESW2(0u) &&
//        STCU2->LBESW != STCU2_LBESW_LBESW3(0u)){
//At least one On line logic BIST partition has completed self test
//Store results from LBSSW register  for later analysis
//Detailed information: Reference manual, section 79.7.12, page 3395
		Online_LBIST_Result = STCU2->LBSSW;
		BistCheckFlag = 1;
    }
//Read On-Line MBIST END Flag registers (MBExSW) to determine if we have completed On-line MBIST
    if (STCU2->MBELSW != 0 || STCU2->MBEMSW != 0){
    	//At least one On line Memory BIST partition has completed self test
		//Store results from LBSSW register  for later analysis
		//Detailed information: Reference manual, section 79.7.25, page 3421
		Online_MBIST_Result_low = STCU2->MBSLSW;
		//Detailed information: Reference manual, section 79.7.23, page 3419
		Online_MBIST_Result_medium = STCU2->MBSMSW;
		//On-Line MBIST Status High Register (STCU2_MBSHSW) all bits reserved, no data.
		BistCheckFlag = 1;
    }
}//STCU2_Online_BIST_check

void STCU2_unlock (void)
{
    STCU2->SKC = 0x753F924E;
    STCU2->SKC = 0x8AC06DB1;
}//STCU2_unlock

void MBIST_Config(void)
{
/* MBIST Config - MBIST Will run first */

/* Configure MBIST Control registers */
    STCU2->MB_CTRL[0] = 0x91000000; /* MBIST CTRL00 Run concurrently, next in sequence is MBIST 1 */
	STCU2->MB_CTRL[1] = 0x92000000; /* MBIST CTRL01 Run concurrently, next in sequence is MBIST 2 */
	STCU2->MB_CTRL[2] = 0x93000000; /* MBIST CTRL02 Run concurrently, next in sequence is MBIST 3 */
	STCU2->MB_CTRL[3] = 0x94000000; /* MBIST CTRL03 Run concurrently, next in sequence is MBIST 4 */
	STCU2->MB_CTRL[4] = 0x95000000; /* MBIST CTRL04 Run concurrently, next in sequence is MBIST 5 */
	STCU2->MB_CTRL[5] = 0x96000000; /* MBIST CTRL05 Run concurrently, next in sequence is MBIST 6 */
	STCU2->MB_CTRL[6] = 0x97000000; /* MBIST CTRL06 Run concurrently, next in sequence is MBIST 7 */
	STCU2->MB_CTRL[7] = 0x98000000; /* MBIST CTRL07 Run concurrently, next in sequence is MBIST 8 */
	STCU2->MB_CTRL[8] = 0x99000000; /* MBIST CTRL08 Run concurrently, next in sequence is MBIST 9 */
	STCU2->MB_CTRL[9] = 0x9A000000; /* MBIST CTRL09 Run concurrently, next in sequence is MBIST 10 */
	STCU2->MB_CTRL[10] = 0x9B000000; /* MBIST CTRL10 Run concurrently, next in sequence is MBIST 11 */
	STCU2->MB_CTRL[11] = 0x9C000000; /* MBIST CTRL11 Run concurrently, next in sequence is MBIST 12 */
	STCU2->MB_CTRL[12] = 0x9D000000; /* MBIST CTRL12 Run concurrently, next in sequence is MBIST 13 */
	STCU2->MB_CTRL[13] = 0x9E000000; /* MBIST CTRL13 Run concurrently, next in sequence is MBIST 14 */
	STCU2->MB_CTRL[14] = 0x9F000000; /* MBIST CTRL14 Run concurrently, next in sequence is MBIST 15 */
	STCU2->MB_CTRL[15] = 0xA0000000; /* MBIST CTRL15 Run concurrently, next in sequence is MBIST 16 */
	STCU2->MB_CTRL[16] = 0xA1000000; /* MBIST CTRL16 Run concurrently, next in sequence is MBIST 17 */
	STCU2->MB_CTRL[17] = 0xA2000000; /* MBIST CTRL17 Run concurrently, next in sequence is MBIST 18 */
	STCU2->MB_CTRL[18] = 0xA3000000; /* MBIST CTRL18 Run concurrently, next in sequence is MBIST 19 */
	STCU2->MB_CTRL[19] = 0xA4000000; /* MBIST CTRL19 Run concurrently, next in sequence is MBIST 20 */
	STCU2->MB_CTRL[20] = 0xA5000000;       /* MBIST CTRL20 Run concurrently, next in sequence is MBIST 21 */

/* Write key 2 to service the watchdog */
    STCU2->SKC = 0x8AC06DB1;

	STCU2->MB_CTRL[21] = 0xA6000000; /* MBIST CTRL21 Run concurrently, next in sequence is MBIST 22 */
	STCU2->MB_CTRL[22] = 0xA7000000; /* MBIST CTRL22 Run concurrently, next in sequence is MBIST 23 */
	STCU2->MB_CTRL[23] = 0xA8000000; /* MBIST CTRL23 Run concurrently, next in sequence is MBIST 24 */
	STCU2->MB_CTRL[24] = 0xA9000000; /* MBIST CTRL24 Run concurrently, next in sequence is MBIST 25 */
	STCU2->MB_CTRL[25] = 0xAA000000; /* MBIST CTRL25 Run concurrently, next in sequence is MBIST 26 */
	STCU2->MB_CTRL[26] = 0xAB000000; /* MBIST CTRL26 Run concurrently, next in sequence is MBIST 27 */
	STCU2->MB_CTRL[27] = 0xAC000000; /* MBIST CTRL27 Run concurrently, next in sequence is MBIST 28 */
	STCU2->MB_CTRL[28] = 0xAD000000; /* MBIST CTRL28 Run concurrently, next in sequence is MBIST 29 */
	STCU2->MB_CTRL[29] = 0xAE000000; /* MBIST CTRL29 Run concurrently, next in sequence is MBIST 30 */
	STCU2->MB_CTRL[30] = 0xAF000000; /* MBIST CTRL30 Run concurrently, next in sequence is MBIST 31 */
	STCU2->MB_CTRL[31] = 0xB0000000; /* MBIST CTRL31 Run concurrently, next in sequence is MBIST 32 */
	STCU2->MB_CTRL[32] = 0xB1000000; /* MBIST CTRL32 Run concurrently, next in sequence is MBIST 33 */
	STCU2->MB_CTRL[33] = 0xB2000000; /* MBIST CTRL33 Run concurrently, next in sequence is MBIST 34 */
	STCU2->MB_CTRL[34] = 0xB3000000; /* MBIST CTRL34 Run concurrently, next in sequence is MBIST 35 */
	STCU2->MB_CTRL[35] = 0xB4000000; /* MBIST CTRL35 Run concurrently, next in sequence is MBIST 36 */
	STCU2->MB_CTRL[36] = 0xB5000000; /* MBIST CTRL36 Run concurrently, next in sequence is MBIST 37 */
	STCU2->MB_CTRL[37] = 0xB6000000; /* MBIST CTRL37 Run concurrently, next in sequence is MBIST 38 */
	STCU2->MB_CTRL[38] = 0xB7000000; /* MBIST CTRL38 Run concurrently, next in sequence is MBIST 39 */

/* Write key 2 to service the watchdog */
    STCU2->SKC = 0x8AC06DB1;

	STCU2->MB_CTRL[39] = 0xB8000000; /* MBIST CTRL39 Run concurrently, next in sequence is MBIST 40 */
	STCU2->MB_CTRL[40] = 0xB9000000; /* MBIST CTRL40 Run concurrently, next in sequence is MBIST 41 */
	STCU2->MB_CTRL[41] = 0xBA000000; /* MBIST CTRL41 Run concurrently, next in sequence is MBIST 42 */
	STCU2->MB_CTRL[42] = 0xBB000000; /* MBIST CTRL42 Run concurrently, next in sequence is MBIST 43 */
	STCU2->MB_CTRL[43] = 0xBC000000; /* MBIST CTRL43 Run concurrently, next in sequence is MBIST 44 */
	STCU2->MB_CTRL[44] = 0xBD000000; /* MBIST CTRL44 Run concurrently, next in sequence is MBIST 45 */
	STCU2->MB_CTRL[45] = 0xBE000000; /* MBIST CTRL45 Run concurrently, next in sequence is MBIST 46 */
	STCU2->MB_CTRL[46] = 0x00000000; /* MBIST CTRL46 Run sequentially, next in sequence is LBIST 0 */

/* Write key 2 to service the watchdog */
    STCU2->SKC = 0x8AC06DB1;
}//Online_BIST

void LBIST_Config (void)
{
/* Configure Online LBIST partition coverage 					*/
/* Configure LBIST for 90% coverage using KEYOFF #define 		*/

    // Write key 2 to service the watchdog
    STCU2->SKC = 0x8AC06DB1;

    //LBIST CTRL0 Run sequentially, next in sequence is LBIST 1
    //0x0103000F
    STCU2->LB[0].CTRL = STCU2_LB_CTRL_CWS(1u)
                      | STCU2_LB_CTRL_PTR(1u)
                      | STCU2_LB_CTRL_SCEN_OFF(1u)
                      | STCU2_LB_CTRL_SCEN_ON(1u)
                      | STCU2_LB_CTRL_SHS(1u);

    /* LBIST 0 pattern count: 0x00000866 */
    STCU2->LB[0].PCS = STCU2_LB_PCS_PCS(0x00000866);
    STCU2->LB[0].MISRELSW = 0xFDC31B19;   /* STCU2 On-Line LBIST MISR Expected Low Register */
    STCU2->LB[0].MISREHSW = 0x18F2A1F5;   /* STCU2 On-Line LBIST MISR Expected High Register */

    /* Write key 2 to service the watchdog */
    STCU2->SKC = 0x8AC06DB1;

    STCU2->LB[1].CTRL = 0x0203000F;       /* LBIST CTRL1 Run sequentially, next in sequence is LBIST 2 */
    STCU2->LB[1].PCS = 0x000007D0;        /* LBIST 1 pattern count */
    STCU2->LB[1].MISRELSW = 0xD8E3627D;   /* LBIST MISREL Expected Low */
    STCU2->LB[1].MISREHSW = 0xAC001775;   /* LBIST MISREH Expected High */

    /* Write key 2 to service the watchdog */
    STCU2->SKC = 0x8AC06DB1;

    STCU2->LB[2].CTRL = 0x0303000F;       /* LBIST CTRL2 Run sequentially, next in sequence is LBIST 3 */
    STCU2->LB[2].PCS = 0x00000834;        /* LBIST 2 pattern count */
    STCU2->LB[2].MISRELSW = 0x4A32B225;   /* LBIST MISREL Expected Low */
    STCU2->LB[2].MISREHSW = 0x765D7C3D;   /* LBIST MISREH Expected High */

    /* Write key 2 to service the watchdog */
    STCU2->SKC = 0x8AC06DB1;

    STCU2->LB[3].CTRL = 0x7F03000F;       /* LBIST CTRL3 Run sequentially */
    STCU2->LB[3].PCS = 0x00000ABE;        /* LBIST 3 pattern count */
    STCU2->LB[3].MISRELSW = 0xEED3E3B7;   /* LBIST MISREL Expected Low */
    STCU2->LB[3].MISREHSW = 0xB489FD55;   /* LBIST MISREH Expected High */

    /* Write key 2 to service the watchdog */
    STCU2->SKC = 0x8AC06DB1;

}//STCU2LBIST_Config

void Online_BIST(void)
{
   STCU2_unlock();

   /* Check and/or clean-up the bit WRP into the STCU2_CFG register to open the IPS access on the STCU2 On-Line Self-Test registers*/
   while((STCU2->CFG && STCU2_CFG_WRP(1u)) == STCU2_CFG_WRP(1u));

   /* Program the Reset management during on-Line LBIST execution setting the STCU2_LBRMSW register */
   //Global functional reset is pulsed at the end of LBISTx run
//   STCU2->LBRMSW = STCU2_LBRMSW_LBRMSW0(1u)
//                 | STCU2_LBRMSW_LBRMSW1(1u)
//                 | STCU2_LBRMSW_LBRMSW2(1u)
//                 | STCU2_LBRMSW_LBRMSW3(1u);
   STCU2->LBRMSW = 0xF;

   /* Overwrite/Program the STCU2_MB_CTRL registers of each NMCUT to be executed */
   MBIST_Config();
   /* Overwrite/Program the STCU2_MB_CTRL registers of each NMCUT to be executed */
   LBIST_Config();

   /* Write key 2 to service the watchdog */
   STCU2->SKC = 0x8AC06DB1;

   /* Set Watchdog timeout Config */
   /* Watchdog time out to Max value */
   STCU2->WDG = 0xFFFFFFFF;

   /* Overwrite/Program the STCU2_CFG register in order to define */
   // Logic, Memory Bist and STCU2 core CLK Clock configuration
   // Start with LBIST_0 pointer (0x0)
   STCU2->CFG = STCU2_CFG_CLK_CFG(0u)
              | STCU2_CFG_MBU(1u)
              | STCU2_CFG_PMOSEN(0u)
              | STCU2_CFG_WRP(0u)
              | STCU2_CFG_PTR(0x10);

//   STCU2.PLL_CFG.B.PLLODF = 2;
//   STCU2.PLL_CFG.B.PLLIDF = 1;
//   STCU2.PLL_CFG.B.PLLLDF = 0x19;

   /* Configure all faults as recoverable in this example */
   STCU2->MBUFML = 0x0;
   /* Configure all faults as recoverable in this example */
   STCU2->MBUFMM = 0x0;

   /* Configure LBIST recoverable/unrecoverable fault handling  */
   STCU2->LBUFM = 0x0;      /* Configure all faults as recoverable in this example */
   /*  Configure LBIST error recoverable/unrecoverable fault handling  */
   STCU2->ERR_FM = 0x0;     /* Configure all faults as recoverable in this example */

   /* Write key 2 to service the watchdog */
   STCU2->SKC = 0x8AC06DB1;

   /* Program the RUNSW bit into the STCU2_RUNSW register to enable the On-Line
      Self-Test execution; the L/MBSWPLLEN bits to enable the PLL lock signal monitor
      during L/MBIST execution and the L/MBIE to enable the related Interrupt requests
    * Start the BIST sequence */
   STCU2->RUNSW = 0x00000301;

   //How to check BIST is complete?
   while(STCU2->RUNSW && STCU2_RUNSW_RUNSW(1u) == STCU2_RUNSW_RUNSW(1u));

}//Online_BIST

//bool IsItCompleteTest(){
//	return(STCU2->RUNSW && STCU2_RUNSW_RUNSW(0u) == STCU2_RUNSW_RUNSW(0u));
//}

/* Function: Flash_Controller_Init
 * Documentation: MPC5746R Reference Manual
 * Follow steps at Platform Flash Configuration Register 1 (PFLASH_PFCR1),
 * section 29.5.1, page 798
 *
 */
void Flash_Controller_Init(void)
{
    // Wait-state settings for 200MHz as system frequency
    // Read Wait State Control
    // Address Pipeline Control
    PFLASH->PFCR1 = PFLASH_PFCR1_RWSC(5u)
                  | PFLASH_PFCR1_APC(2u);
}//Flash_Controller_Init

/* Function: FCCU_clear_faults
 * Documentation: MPC5746R Reference Manual
 * Follow steps at Noncritical Fault Status (FCCU_NCF_Sn), section 78.7.7, page 3319
 */
void FCCU_clear_faults(void)
{
  //1. Write the proper key into the FCCU_NCFK register
  //Non-critical fault key = AB34_98FEh
  FCCU->NCFK = 0xAB3498FE;

  //2. Clear the status (flag) bit NCFSx => the opcode OP12 is automatically set into the FCCU_CTRL.OPR field
  //Read all NCFS registers to clear all faults
  //For details which faults can be cleared see FCCU Non-Critical Faults Mapping in reference manual
  FCCU->NCF_S[0] = 0xFFFFFFFF;		// read FCCU.NCF_S0 register

  //Verify if state change was successful
  while(FCCU->CTRL && FCCU_CTRL_OPS(0x3) != FCCU_CTRL_OPS(0x3));	//Operation status successful

  //NCFS_1 register clear
  FCCU->NCFK = 0xAB3498FE;		//Non-critical fault key = AB34_98FEh
  FCCU->NCF_S[1] = 0xFFFFFFFF;		// clear FCCU.NCF_S1 register
  //Verify if state change was successful
  while(FCCU->CTRL && FCCU_CTRL_OPS(0x3) != FCCU_CTRL_OPS(0x3));	//Operation status successful

  //NCFS_2 register clear
  FCCU->NCFK = 0xAB3498FE;		//Non-critical fault key = AB34_98FEh
  FCCU->NCF_S[2] = 0xFFFFFFFF;		// clear FCCU.NCF_S2 register
  //Verify if state change was successful
  //Operation Status(OPS): 00->Idle, 01->In progress, 10->Aborted, 11->Successful
  while(FCCU->CTRL && FCCU_CTRL_OPS(0x3) != FCCU_CTRL_OPS(0x3));	//Operation status-> (0b11 or 0x3)successful
}//FCCU_clear_faults

/* Function list to execute Online BIST */

void RunOnlineBist (void)
{
	FCCU_clear_faults();
	Flash_Controller_Init();
	Sys_Init();
	SIUL2_Init();
	STCU2_Online_BIST_check();    	//Check online BIST results
	if(BistCheckFlag == 0){
	    //Configure LBIST and MBIST
	    Online_BIST();              //Start online BIST
	}
	while(1);
}//main
